@@ -1,27 +1,29 @@
 # This file is part of Wike (com.github.hugolabe.Wike)
-# SPDX-FileCopyrightText: 2021-23 Hugo Olabera <hugolabe@gmail.com>
+# SPDX-FileCopyrightText: 2021-24 Hugo Olabera <hugolabe@gmail.com>
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 
 from datetime import datetime
 
-from gi.repository import GObject, Gio, Gtk, Adw
+from gi.repository import GObject, Gio, Gtk, Adw, Pango
 
 from wike.data import settings, languages, history
 
 
-# Box for history in sidebar
+# History panel for sidebar
 
-@Gtk.Template(resource_path='/com/github/hugolabe/Wike/ui/history.ui')
-class HistoryBox(Gtk.Box):
+@Gtk.Template(resource_path='/com/github/hugolabe/Wike/gtk/history.ui')
+class HistoryPanel(Adw.Bin):
 
-  __gtype_name__ = 'HistoryBox'
+  __gtype_name__ = 'HistoryPanel'
 
   filter_dropdown = Gtk.Template.Child()
+  selection_button = Gtk.Template.Child()
   clear_button = Gtk.Template.Child()
+  select_all_button = Gtk.Template.Child()
+  select_none_button = Gtk.Template.Child()
+  remove_button = Gtk.Template.Child()
   history_list = Gtk.Template.Child()
-  history_scroller = Gtk.Template.Child()
-  history_separator = Gtk.Template.Child()
 
   # Initialize widgets and connect signals and bindings
 
@@ -49,14 +51,17 @@ class HistoryBox(Gtk.Box):
     self._populate()
 
     self.clear_button.connect('clicked', self._clear_button_cb)
+    self.select_all_button.connect('clicked', self._select_buttons_cb, True)
+    self.select_none_button.connect('clicked', self._select_buttons_cb, False)
+    self.remove_button.connect('clicked', self._remove_button_cb)
     self.history_list.connect('row-activated', self._list_activated_cb)
-    self.history_scroller.get_vadjustment().connect('value-changed', self._history_scrolled_cb)
 
   # Setup filter item with a label
 
   def _filter_factory_setup(self, factory, list_item):
     label = Gtk.Label()
     label.set_xalign(0)
+    label.set_ellipsize(Pango.EllipsizeMode.END)
     list_item.set_child(label)
 
   # Bind label property for filter item
@@ -70,7 +75,6 @@ class HistoryBox(Gtk.Box):
 
   def _populate(self):
     self._clear_list()
-    self.history_scroller.get_vadjustment().set_value(0)
 
     history_filter = self.filter_dropdown.get_selected_item()
     filter_days = history_filter.days
@@ -83,7 +87,8 @@ class HistoryBox(Gtk.Box):
         break
 
       date_label = Gtk.Label()
-      date_label.set_markup('<b>' + date + '</b>')
+      date_label.set_label(date)
+      date_label.add_css_class('heading')
       date_label.set_xalign(0)
       date_label.set_margin_start(3)
       date_label.set_margin_end(3)
@@ -100,11 +105,13 @@ class HistoryBox(Gtk.Box):
         lang = history_item[uri][2]
         row = HistoryRow(uri, title, lang, date, time)
         self.history_list.append(row)
-        row.remove_button.connect('clicked', self._row_remove_button_cb, row)
+        self.selection_button.bind_property('active', row.select_check, 'visible', Gio.SettingsBindFlags.DEFAULT)
 
   # Clear history list
 
   def _clear_list(self):
+    self.selection_button.set_active(False)
+
     while True:
       row = self.history_list.get_row_at_index(0)
       if row:
@@ -142,7 +149,7 @@ class HistoryBox(Gtk.Box):
 
   def _clear_button_cb(self, clear_button):
     builder = Gtk.Builder()
-    builder.add_from_resource('/com/github/hugolabe/Wike/ui/dialogs.ui')
+    builder.add_from_resource('/com/github/hugolabe/Wike/gtk/dialogs.ui')
     clear_history_dialog = builder.get_object('clear_history_dialog')
     clear_history_dialog.set_transient_for(self._window)
 
@@ -156,17 +163,41 @@ class HistoryBox(Gtk.Box):
     if response == 'clear':
       self.clear_history()
 
-  # On list activated load article in view
+  # On select all or select none buttons do selection
+  
+  def _select_buttons_cb(self, button, select_mode):
+    i = 0
+    while True:
+      row = self.history_list.get_row_at_index(i)
+      if row:
+        if row.get_activatable():
+          row.select_check.set_active(select_mode)
+        i += 1
+      else:
+        break
 
-  def _list_activated_cb(self, history_list, row):
-    if self._window.flap.get_fold_policy() == Adw.FlapFoldPolicy.ALWAYS:
-      self._window.flap.set_reveal_flap(False)
+  # On button clicked remove selected items
 
-    self._window.page.wikiview.load_wiki(row.uri)
+  def _remove_button_cb(self, button):
+    selected_rows = []
+    i = 0
+    while True:
+      row = self.history_list.get_row_at_index(i)
+      if row:
+        if row.get_activatable():
+          if row.select_check.get_active():
+            selected_rows.append(row)
+        i += 1
+      else:
+        break
+        
+    if len(selected_rows) > 0:
+      for row in selected_rows:
+        self._remove_row(row)
 
-  # On row button remove article from history
+  # Remove row from history and list
 
-  def _row_remove_button_cb(self, remove_button, row):
+  def _remove_row(self, row):
     item_deleted, date_deleted = history.remove(row.date, row.uri)
     row_index = row.get_index()
 
@@ -177,13 +208,17 @@ class HistoryBox(Gtk.Box):
       date_row = self.history_list.get_row_at_index(row_index-1)
       self.history_list.remove(date_row)
 
-  # Show/hide separator on scroll
+  # On list activated load article in view
 
-  def _history_scrolled_cb(self, adjustment):
-    if adjustment.get_value() > 0:
-      self.history_separator.set_visible(True)
-    else:
-      self.history_separator.set_visible(False)
+  def _list_activated_cb(self, history_list, row):
+    if self.selection_button.get_active():
+      row.select_check.set_active(not row.select_check.get_active())
+      return
+
+    if self._window.panel_split.get_collapsed():
+      self._window.panel_split.set_show_sidebar(False)
+
+    self._window.page.wikiview.load_wiki(row.uri)
 
 
 # This object represent an history filter in dropdown
@@ -214,7 +249,7 @@ class HistoryFilter(GObject.Object):
 
 # Row on history list
 
-@Gtk.Template(resource_path='/com/github/hugolabe/Wike/ui/history-row.ui')
+@Gtk.Template(resource_path='/com/github/hugolabe/Wike/gtk/history-row.ui')
 class HistoryRow(Gtk.ListBoxRow):
 
   __gtype_name__ = 'HistoryRow'
@@ -222,7 +257,7 @@ class HistoryRow(Gtk.ListBoxRow):
   title_label = Gtk.Template.Child()
   lang_label = Gtk.Template.Child()
   time_label = Gtk.Template.Child()
-  remove_button = Gtk.Template.Child()
+  select_check = Gtk.Template.Child()
 
   # Set values and widgets
 
@@ -241,5 +276,5 @@ class HistoryRow(Gtk.ListBoxRow):
       lang_name = lang
 
     self.title_label.set_label(title)
-    self.lang_label.set_markup('<small>' + lang_name + '</small>')
-    self.time_label.set_markup('<small>' + self.time + '</small>')
+    self.lang_label.set_label(lang_name)
+    self.time_label.set_label(self.time)
